@@ -28,12 +28,12 @@
 #include "GormFontViewController.h"
 #include "GormSetNameController.h"
 #include "GNUstepGUI/GSNibCompatibility.h"
+#include "GNUstepBase/GSObjCRuntime.h"
 
 // for templates...
 #include <AppKit/NSControl.h>
 #include <AppKit/NSButton.h>
 
-NSDate	*startDate;
 NSString *GormLinkPboardType = @"GormLinkPboardType";
 NSString *GormToggleGuidelineNotification = @"GormToggleGuidelineNotification";
 NSString *GormDidModifyClassNotification = @"GormDidModifyClassNotification";
@@ -42,6 +42,12 @@ NSString *GormDidDeleteClassNotification = @"GormDidDeleteClassNotification";
 NSString *GormWillDetachObjectFromDocumentNotification = @"GormWillDetachObjectFromDocumentNotification";
 NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 
+// Define this as "NO" initially.   We only want to turn this on while loading or testing.
+static BOOL _isInInterfaceBuilder = NO;
+
+static NSImage *gormImage = nil;
+static NSImage *testingImage = nil;
+
 @class	InfoPanel;
 
 // we had this include for grouping/ungrouping selectors
@@ -65,9 +71,9 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
     }
   else
     {
-      /* FIXME/TODO.  What do we do if we are an attributed string.  
+      /* TODO: What do we do if we are an attributed string.  
          Think about what happens when the user ends editing. 
-         Allows editing text attributes... Formatter... TODO. */
+         Allows editing text attributes... Formatter. */
     }
   [textObject setEditable: _cell.is_editable];
   [textObject setSelectable: _cell.is_selectable || _cell.is_editable];
@@ -254,6 +260,10 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
       sourceImage = [[NSImage alloc] initWithContentsOfFile: path];
       path = [bundle pathForImageResource: @"GormTargetTag"];
       targetImage = [[NSImage alloc] initWithContentsOfFile: path];
+      path = [bundle pathForImageResource: @"Gorm"];
+      gormImage = [[NSImage alloc] initWithContentsOfFile: path];
+      path = [bundle pathForImageResource: @"GormTesting"];
+      testingImage = [[NSImage alloc] initWithContentsOfFile: path];
 
       documents = [NSMutableArray new];
       [nc addObserver: self
@@ -300,6 +310,7 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 {
   NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
   NSArray *a = nil;
+  
 
   if ( [defaults boolForKey: @"ShowInspectors"] )
     {
@@ -316,6 +327,13 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 	  NSLog(@"WARNING: Gorm has detected that you are using user bundles.  Please make certain that these are compatible with Gorm as some bundles can cause issues which may corrupt your .gorm files.");
 	}
     }
+  if(GSGetMethod([GSNibContainer class],@selector(awakeWithContext:),YES,YES) == NULL)
+    {
+      NSRunAlertPanel(_(@"Incorrect GNUstep Version"), 
+		      _(@"The version of GNUstep you are using is too old for this version of Gorm, please update."),
+		      _(@"OK"), NULL, NULL);
+      [self terminate: self];
+    }
 }
 
 
@@ -642,7 +660,6 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
   GormDocument  *document = (GormDocument *)[self activeDocument];
   NSWindow	*window = [document window];
 
-  [window setReleasedWhenClosed: YES];
   [window performClose: self];
 }
 
@@ -679,6 +696,7 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 	  
 	  
 	  isTesting = YES; // set here, so that beginArchiving and endArchiving do not use templates.
+	  [self setApplicationIconImage: testingImage];
 	  archiver = [[NSArchiver alloc] init];
 	  [activDoc beginArchiving];
 	  [archiver encodeClassName: @"GormCustomView" 
@@ -701,10 +719,12 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 	  [archiver encodeClassName: @"GormNSOutlineView" 
 		    intoClassName: @"NSOutlineView"];
 	  */
+	  [GSClassSwapper setIsInInterfaceBuilder: YES]; // do not allow custom classes during testing.
 	  [archiver encodeRootObject: activDoc];
 	  data = RETAIN([archiver archiverData]); // Released below... 
 	  [activDoc endArchiving];
 	  RELEASE(archiver);
+	  [GSClassSwapper setIsInInterfaceBuilder: NO]; // beginal allowing custom classes...
 	  
 	  [notifCenter postNotificationName: IBWillBeginTestingInterfaceNotification
 		       object: self];
@@ -722,8 +742,10 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 	  testContainer = [NSUnarchiver unarchiveObjectWithData: data];
 	  if (testContainer != nil)
 	    {
-	      [testContainer awakeWithContext: nil
-			     topLevelItems: nil]; // FIXME: Top level items will leak...
+	      NSMutableDictionary *nameTable = [testContainer nameTable];
+	      [nameTable removeObjectForKey: @"NSServicesMenu"];
+	      [nameTable removeObjectForKey: @"NSWindowsMenu"];
+	      [testContainer awakeWithContext: nil];
 	      RETAIN(testContainer); // released in endTesting:
 	    }
 	  
@@ -735,8 +757,8 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 	    {
 	      NSMenu	*testMenu;
 	      
-	      testMenu = [[NSMenu alloc] initWithTitle: _(@"Test Menu")];
-	      [testMenu addItemWithTitle: _(@"Quit") 
+	      testMenu = [[NSMenu alloc] initWithTitle: _(@"Test Menu (Gorm)")];
+	      [testMenu addItemWithTitle: _(@"Quit Test") 
 			action: @selector(deferredEndTesting:)
 			keyEquivalent: @"q"];	
 	      [self setMainMenu: testMenu]; // released, when the menu is reset in endTesting.
@@ -744,16 +766,20 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 	  else
 	    {
 	      NSMenu	*testMenu = [self mainMenu];
-	      id		item;
-	      
+	      id	 item;
+	      NSString  *newTitle = [[testMenu title] stringByAppendingString: @" (Gorm)"];
+
+	      // set the menu up so that it's easy to tell we're testing and how to quit.
+	      [testMenu setTitle: newTitle];
 	      item = [testMenu itemWithTitle: _(@"Quit")];
 	      if (item != nil)
 		{
+		  [item setTitle: _(@"Quit Test")];
 		  [item setAction: @selector(deferredEndTesting:)];
 		}
 	      else
 		{
-		  [testMenu addItemWithTitle: _(@"Quit") 
+		  [testMenu addItemWithTitle: _(@"Quit Test") 
 			    action: @selector(deferredEndTesting:)
 			    keyEquivalent: @"q"];	
 		}
@@ -775,7 +801,7 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 	  // reset the application after the error.
 	  NSLog(@"Error while testing interface: %@", 
 		[localException reason]);
-	  NSRunAlertPanel(NULL, [NSString stringWithFormat: @"Problem testing interface.  Make sure connections are to appropriate objects.  Exception: %@",
+	  NSRunAlertPanel(_(@"An Error Occurred"), [NSString stringWithFormat: @"Problem testing interface.  Make sure connections are to appropriate objects.  Exception: %@",
 					  [localException reason]], 
 			  _(@"OK"), NULL, NULL);
 	  [self endTesting: self];
@@ -828,7 +854,7 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 
 - (void) selectAllItems: (id)sender
 {
-  /* FIXME */
+  /* TODO: Select all items in the current selection owner. */
   return;
 }
 
@@ -920,18 +946,13 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
   [(GormGenericEditor *)selectionOwner ungroup];
 }
 
-
-
-/***********************************************************************/
-/***********************   Classes Action  *******************************/
-/***********************************************************************/
+/// Classes actions...
 
 - (void) createSubclass: (id)sender
 {
   [(GormDocument *)[self activeDocument] createSubclass: sender];
 }
 
-
 - (void) loadClass: (id)sender
 {
   // Call the current document and create the class 
@@ -959,18 +980,7 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
   [(GormDocument *)[self activeDocument] remove: sender];
 }
 
-/*
-- (id) editClass: (id)sender
-{
-  [self inspector: self];
-  return [(id)[self activeDocument] editClass: sender];
-}
-*/
-
-
-/***********************************************************************/
-/***********************   Classes Action  *******************************/
-/***********************************************************************/
+/// Palettes Actions...
 
 - (void) inspector: (id) sender
 {
@@ -987,6 +997,7 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
   [[self palettesManager] openPalette: sender];
 }
 
+/// Testing methods...
 
 - (void) deferredEndTesting: (id) sender
 {
@@ -1039,6 +1050,7 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 	}
 
       [self setMainMenu: mainMenu];
+      [self setApplicationIconImage: gormImage];
 
       NS_DURING
 	{
@@ -1048,8 +1060,8 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 	{
 	  NSDebugLog(@"Exception while setting services menu");
 	}
-
       NS_ENDHANDLER
+
       [mainMenu display]; // bring it to the front...
 
       isTesting = NO;
@@ -1092,7 +1104,6 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 
   [self setDelegate: self];
   [super finishLaunching];
-  NSDebugLog(@"StartupTime %f", [startDate timeIntervalSinceNow]);
 }
 
 - (void) handleNotification: (NSNotification*)notification
@@ -1239,6 +1250,13 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
   GormDocument	*active = (GormDocument*)[self activeDocument];
   SEL		action = [item action];
   GormClassManager *cm = [active classManager];
+  NSArray	*s = [selectionOwner selection];
+
+  // temporarily disabling this functionality....
+  if (sel_eq(action, @selector(loadClass:)))
+    {
+      return NO;
+    }
 
   if (sel_eq(action, @selector(close:))
     || sel_eq(action, @selector(miniaturize:))
@@ -1262,30 +1280,71 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
     }
   else if (sel_eq(action, @selector(copy:)))
     {
-      if ([[selectionOwner selection] count] == 0)
+      if ([s count] == 0)
 	return NO;
+      else
+	{
+	  id	    o = [s objectAtIndex: 0];
+	  NSString *n = [active nameForObject: o];
+	  if ([n isEqual: @"NSOwner"] || [n isEqual: @"NSFirst"])
+	    {
+	      return NO;
+	    }
+	}
+
       return [selectionOwner respondsToSelector: @selector(copySelection)];
     }
   else if (sel_eq(action, @selector(cut:)))
     {
-      if ([[selectionOwner selection] count] == 0)
+      if ([s count] == 0)
 	return NO;
+      else
+	{
+	  id	    o = [s objectAtIndex: 0];
+	  NSString *n = [active nameForObject: o];
+	  if ([n isEqual: @"NSOwner"] || [n isEqual: @"NSFirst"])
+	    {
+	      return NO;
+	    }
+	}
+
       return ([selectionOwner respondsToSelector: @selector(copySelection)]
 	&& [selectionOwner respondsToSelector: @selector(deleteSelection)]);
     }
   else if (sel_eq(action, @selector(delete:)))
     {
-      if ([[selectionOwner selection] count] == 0)
+      if ([s count] == 0)
 	return NO;
+      else
+	{
+	  id	    o = [s objectAtIndex: 0];
+	  NSString *n = [active nameForObject: o];
+	  if ([n isEqual: @"NSOwner"] || [n isEqual: @"NSFirst"])
+	    {
+	      return NO;
+	    }
+	}
+
       return [selectionOwner respondsToSelector: @selector(deleteSelection)];
     }
   else if (sel_eq(action, @selector(paste:)))
     {
+      if ([s count] == 0)
+	return NO;
+      else
+	{
+	  id	o = [s objectAtIndex: 0];
+	  NSString *n = [active nameForObject: o];
+	  if ([n isEqual: @"NSOwner"] || [n isEqual: @"NSFirst"])
+	    {
+	      return NO;
+	    }
+	}
+
       return [selectionOwner respondsToSelector: @selector(pasteInSelection)];
     }
   else if (sel_eq(action, @selector(setName:)))
     {
-      NSArray	*s = [selectionOwner selection];
       NSString	*n;
       id	o;
 
@@ -1352,7 +1411,6 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 	 sel_eq(action, @selector(createClassFiles:)) || 
 	 sel_eq(action, @selector(remove:)))
 	{
-	  NSArray *s = [selectionOwner selection];
 	  id o = nil;
 	  NSString *name = nil;
 
@@ -1378,7 +1436,6 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 
       if(sel_eq(action, @selector(instantiateClass:)))
 	{
-	  NSArray *s = [selectionOwner selection];
 	  id o = nil;
 	  NSString *name = nil;
 
@@ -1458,14 +1515,15 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 @end
 
 // custom class additions...
-@interface GSClassSwapper (GormCustomClassAdditions)
-- (BOOL) isInInterfaceBuilder;
-@end
-
 @implementation GSClassSwapper (GormCustomClassAdditions)
++ (void) setIsInInterfaceBuilder: (BOOL)flag
+{
+  _isInInterfaceBuilder = flag;
+}
+
 - (BOOL) isInInterfaceBuilder
 {
-  return YES;
+  return _isInInterfaceBuilder;
 }
 @end
 
@@ -1546,11 +1604,4 @@ NSString *GormResizeCellNotification = @"GormResizeCellNotification";
 // END OF DEPRECATED TEMPLATES                        //
 ////////////////////////////////////////////////////////
 
-// main...
-int 
-main(int argc, const char **argv)
-{ 
-  startDate = [[NSDate alloc] init];
-  return NSApplicationMain(argc, argv);
-}
 
