@@ -26,7 +26,10 @@
 #include "GormPrivate.h"
 #include "GormCustomView.h"
 #include "GormDocument.h"
+#include "GormFilesOwner.h"
+#include "GormPalettesManager.h"
 #include <InterfaceBuilder/IBEditors.h>
+#include <InterfaceBuilder/IBPalette.h>
 
 @interface	GormClassManager (Private)
 - (NSMutableDictionary*) classInfoForClassName: (NSString*)className;
@@ -127,6 +130,19 @@
    withSuperClassNamed: (NSString*)super_class_name
 	   withActions: (NSArray*)_actions
 	   withOutlets: (NSArray*)_outlets
+{
+  return [self addClassNamed: class_name
+	       withSuperClassNamed: super_class_name
+	       withActions: _actions
+	       withOutlets: _outlets
+	       isCustom: YES];
+}
+
+- (BOOL) addClassNamed: (NSString*)class_name
+   withSuperClassNamed: (NSString*)super_class_name
+	   withActions: (NSArray*)_actions
+	   withOutlets: (NSArray*)_outlets
+	      isCustom: (BOOL) isCustom
 {
   BOOL result = NO;
   NSString *className = [class_name copy];
@@ -151,12 +167,17 @@
 	  [classInfo setObject: actions forKey: @"Actions"];
 	  [classInfo setObject: superClassName forKey: @"Super"];
 	  [classInformation setObject: classInfo forKey: className];
-	  [customClasses addObject: className];
+	  
+	  // if it's a custom class add it to the list.
+	  if(isCustom)
+	    {
+	      [customClasses addObject: className];
+	    }
 
 	  // copy all actions from the class imported to the first responder
 	  while((action = [e nextObject]))
 	    {
-	      if([self isSuperclass: @"NSResponder" linkedToClass: className])
+	      // if([self isSuperclass: @"NSResponder" linkedToClass: className])
 		{
 		  [self addAction: action forClassNamed: @"FirstResponder"];
 		}
@@ -208,7 +229,7 @@
   [[info objectForKey: @"AllActions"] insertObject: anAction atIndex: 0];
   if(![className isEqualToString: @"FirstResponder"]) 
     {
-      if([self isSuperclass: @"NSResponder" linkedToClass: className])
+      // if([self isSuperclass: @"NSResponder" linkedToClass: className])
 	{
 	  [self addAction: anAction forClassNamed: @"FirstResponder"];
 	}
@@ -303,7 +324,7 @@
 
   if(![className isEqualToString: @"FirstResponder"]) 
     {
-      if([self isSuperclass: @"NSResponder" linkedToClass: className])
+      // if([self isSuperclass: @"NSResponder" linkedToClass: className])
 	{
 	  [self replaceAction: oldAction withAction: newAction forClassNamed: @"FirstResponder"];
 	}
@@ -397,7 +418,7 @@
 
   if(![className isEqualToString: @"FirstResponder"]) 
     {
-      if([self isSuperclass: @"NSResponder" linkedToClass: className])
+      // if([self isSuperclass: @"NSResponder" linkedToClass: className])
 	{
 	  [self removeAction: anAction fromClassNamed: @"FirstResponder"];
 	}
@@ -818,6 +839,8 @@
 	}
       else
 	{
+	  GormPalettesManager *palettesManager = [(Gorm *)NSApp palettesManager];
+
 	  // load the classes, initialize the custom class array and map..
 	  [self loadFromFile: path];
 	  customClasses = [[NSMutableArray alloc] initWithCapacity: 1];
@@ -825,6 +848,9 @@
 	  
 	  // add first responder so that it may be edited.
 	  [customClasses addObject: @"FirstResponder"];
+	  
+	  // add the imported classes to the class information list...
+	  [classInformation addEntriesFromDictionary: [palettesManager importedClasses]];
 	}
     }
   
@@ -927,6 +953,7 @@
     {
       NSEnumerator *en = [customClassMap keyEnumerator];
       id object = nil;
+      id owner = nil;
 
       [customClasses removeObject: className];
       
@@ -942,6 +969,13 @@
 		}
 	    }
 	}
+
+      // get the owner and reset the class name to NSApplication.
+      owner = [document objectForName: @"NSOwner"];
+      if([className isEqual: [owner className]])
+	{
+	  [owner setClassName: @"NSApplication"];
+	}
     }
 
   [classInformation removeObjectForKey: className];
@@ -1348,269 +1382,15 @@
 
 - (BOOL) parseHeader: (NSString *)headerPath
 {
-  NSString *headerFile = [NSString stringWithContentsOfFile: headerPath];
-  NSScanner *headerScanner = [NSScanner scannerWithString: headerFile];
-  NSCharacterSet *superClassStopSet = [NSCharacterSet characterSetWithCharactersInString: @" \n"];
-  NSCharacterSet *classStopSet = [NSCharacterSet characterSetWithCharactersInString: @" :"];
-  NSCharacterSet *categoryStopSet = [NSCharacterSet characterSetWithCharactersInString: @" ("];
-  NSCharacterSet *typeStopSet = [NSCharacterSet characterSetWithCharactersInString: @" "];
-  NSCharacterSet *actionStopSet = [NSCharacterSet characterSetWithCharactersInString: @";:"];
-  NSCharacterSet *outletStopSet = [NSCharacterSet characterSetWithCharactersInString: @";,"];
-  NSCharacterSet *illegalOutletSet = [NSCharacterSet characterSetWithCharactersInString: @"~`@#$%^&*()+={}|[]\\:;'<>?,./"];
-  NSCharacterSet *illegalActionSet = [NSCharacterSet characterSetWithCharactersInString: @"~`@#$%^&*()+={}|[]\\;'<>?,./"];
-  NSArray *outletTokens = [NSArray arrayWithObjects: @"id", @"IBOutlet", nil];
-  NSArray *actionTokens = [NSArray arrayWithObjects: @"(void)", @"(IBAction)", @"(id)", nil];
-  NSRange notFoundRange = NSMakeRange(NSNotFound,0);
-  // NSCharacterSet *commentStopSet = [NSCharacterSet characterSetWithCharactersInString: @"\n"];
+  BOOL result = NO;
+  /*
+  OCHeaderParser *ochp = [[OCHeaderParser alloc] initWithContentsOfFile: headerPath];
+  NSArray *classes = [ochp ];
+  NSArray *actions = [ochp actionList];
+  NSString 
+  */
 
-  while (![headerScanner isAtEnd])
-    {
-      NSString *classString = nil;
-      BOOL classfound = NO, result = NO, category = NO;
-      NSEnumerator *outletEnum = [outletTokens objectEnumerator];
-      NSEnumerator *actionEnum = [actionTokens objectEnumerator];
-      NSString *outletToken = nil;
-      NSString *actionToken = nil;
-      int alert;
-
-      classfound = [headerScanner scanUpToString: @"@interface"
-			     intoString: NULL];
-
-      [headerScanner scanUpToString: @"@end"
-		     intoString: &classString];
-      
-      if (classfound && ![headerScanner isAtEnd])
-	{
-	  NSString 
-	    *className = nil,
-	    *superClassName = nil,
-	    *ivarString = nil,
-	    *methodString = nil;
-	  NSScanner 
-	    *classScanner = [NSScanner scannerWithString: classString],
-	    *ivarScanner = nil,
-	    *methodScanner = nil;
-	  NSMutableArray 
-	    *actions = [NSMutableArray array],
-	    *outlets = [NSMutableArray array];
-
-	  [classScanner scanString: @"@interface"
-			intoString: NULL];
-	  [classScanner scanUpToCharactersFromSet: classStopSet
-			intoString: &className];
-	  [classScanner scanString: @":"
-			intoString: NULL];
-	  [classScanner scanUpToCharactersFromSet: superClassStopSet
-			intoString: &superClassName];
-	  [classScanner scanUpToString: @"{"
-			intoString: NULL];
-	  [classScanner scanUpToString: @"}"
-			intoString: &ivarString];
-
-	  category = (ivarString == nil);
-	  if(!category)
-	    {
-	      [classScanner scanUpToString: @"@end"
-			    intoString: &methodString];
-	      NSDebugLog(@"Found a class \"%@\" with super class \"%@\"", className,
-			 superClassName);
-	    }
-	  else
-	    {
-	      NSDebugLog(@"A CATEGORY");
-	      classScanner = [NSScanner scannerWithString: classString];
-	      [classScanner scanString: @"@interface"
-			    intoString: NULL];
-	      [classScanner scanUpToCharactersFromSet: categoryStopSet
-			    intoString: &className];
-	      [classScanner scanString: @"("
-			    intoString: NULL];
-	      [classScanner scanUpToCharactersFromSet: superClassStopSet
-			    intoString: &superClassName];
-	      [classScanner scanString: @")"
-			    intoString: NULL];
-	      [classScanner scanUpToString: @"@end"
-			    intoString: &methodString];
-	      NSDebugLog(@"method String %@",methodString);
-	    }
-
-
-	  // if its' not a category and it's known, ask before proceeding...
-	  if([self isKnownClass: className] && !category)
-	    {
-	      NSString *message = [NSString stringWithFormat: 
-					      _(@"The class %@ already exists. Replace it?"), 
-					    className];
-	      alert = NSRunAlertPanel(_(@"Problem adding class from header"), 
-				      message,
-				      _(@"Yes"), 
-				      _(@"No"), 
-				      nil);
-	      if (alert != NSAlertDefaultReturn)
-		return NO;
-	    }
-	  
-	  // if it's not a category go through the ivars...
-	  if(!category)
-	    {
-	      NSDebugLog(@"Ivar string is not nil");
-	      // Interate over the possible tokens which can make an
-	      // ivar an outlet.
-	      while ((outletToken = [outletEnum nextObject]) != nil)
-		{
-		  NSString *delimiter = nil;
-		  NSDebugLog(@"outlet Token = %@", outletToken);
-		  // Scan the variables of the class...
-		  ivarScanner = [NSScanner scannerWithString: ivarString];
-		  while (![ivarScanner isAtEnd])
-		    {
-		      NSString *outlet = nil;
-		      NSString *type = nil;
-		      
-		      if (delimiter == nil || [delimiter isEqualToString: @";"])
-			{
-			  [ivarScanner scanUpToString: outletToken
-				       intoString: NULL];
-			  [ivarScanner scanString: outletToken
-				       intoString: NULL];
-			}
-		      
-		      // if using the IBOutlet token in the header, scan in the outlet type
-		      // as well.
-		      if([outletToken isEqualToString: @"IBOutlet"])
-			{
-			  [ivarScanner scanUpToCharactersFromSet: typeStopSet
-				       intoString: NULL];
-			  [ivarScanner scanCharactersFromSet: typeStopSet
-				       intoString: NULL];
-			  [ivarScanner scanUpToCharactersFromSet: typeStopSet
-				       intoString: &type];
-			  NSDebugLog(@"outlet type = %@",type);
-			}
-
-		      [ivarScanner scanUpToCharactersFromSet: outletStopSet
-				   intoString: &outlet];
-		      [ivarScanner scanCharactersFromSet: outletStopSet
-				   intoString: &delimiter];
-		      if ([ivarScanner isAtEnd] == NO
-			  && [outlets indexOfObject: outlet] == NSNotFound)
-			{
-			  NSDebugLog(@"outlet = %@", outlet);
-			  if(NSEqualRanges([outlet rangeOfCharacterFromSet: illegalOutletSet],notFoundRange))
-			    {
-			      [outlets addObject: outlet];
-			    }
-			}
-		    }
-		}
-	    }
-	  
-	  while ((actionToken = [actionEnum nextObject]) != nil)
-	    {
-	      NSDebugLog(@"Action token %@", actionToken);
-	      methodScanner = [NSScanner scannerWithString: methodString];
-	      while (![methodScanner isAtEnd])
-		{
-		  NSString *action = nil;
-		  BOOL hasArguments = NO;
-		  
-		  // Scan the method name
-		  [methodScanner scanUpToString: actionToken
-				 intoString: NULL];
-		  [methodScanner scanString: actionToken
-				 intoString: NULL];
-		  [methodScanner scanUpToCharactersFromSet: actionStopSet
-				 intoString: &action];
-		  
-		  // This will return true if the method has args.
-		  hasArguments = [methodScanner scanString: @":"
-						intoString: NULL];
-		  
-		  if (hasArguments)
-		    {
-		      BOOL isAction = NO;
-		      NSString *argType = nil;
-		      
-		      // If the argument is (id) then the method can
-		      // be considered an action and we add it to the list.
-		      isAction = [methodScanner scanString: @"(id)"
-						intoString: &argType];
-		      
-		      if (![methodScanner isAtEnd])
-			{
-			  if (isAction)
-			    {
-			      /* Add the ':' back */
-			      action = [action stringByAppendingString: @":"];
-			      NSDebugLog(@"action = %@", action);
-			      if(NSEqualRanges([action rangeOfCharacterFromSet: illegalActionSet],notFoundRange))
-				{
-				  [actions addObject: action];
-				}
-			    }
-			  else
-			    {
-			      NSDebugLog(@"Not an action");
-			    }
-			}
-		    }
-		} // end while
-	    } // end while 
-
-	  if([self isKnownClass: className] && 
-	     [self isCustomClass: className] && category) 
-	    {
-	      [self addActions: actions forClassNamed: className];
-	      [self addOutlets: outlets forClassNamed: className];
-	      result = YES;
-	    }
-	  else if(!category)
-	    {
-	      result = [self addClassNamed: className
-			     withSuperClassNamed: superClassName
-			     withActions: actions
-			     withOutlets: outlets];
-	    }
-
-	  if (result)
-	    {
-	      NSDebugLog(@"Class %@ added", className);
-	    }
-	  else
-	    if (alert == NSAlertDefaultReturn)
-	      {
-		[self removeClassNamed: className];
-		result = [self addClassNamed: className
-			       withSuperClassNamed: superClassName
-			       withActions: actions
-			       withOutlets: outlets];
-		if (!result)
-		  {
-		    NSString *message = [NSString stringWithFormat: 
-						    _(@"Could not replace class %@."), className];	      
-		    NSRunAlertPanel(_(@"Problem adding class from header"), 
-				      message,
-				    nil, 
-				    nil, 
-				    nil);
-		    NSDebugLog(@"Class %@ failed to add", className);
-		  }
-		else
-		  {
-		    NSDebugLog(@"Class %@ replaced.", className);
-		  }
-	      }
-	   
-	  if (result)
-	    {
-	      [[NSNotificationCenter defaultCenter] 
-		postNotificationName: GormDidAddClassNotification
-		object: self];
-	      [document selectClass: className];
-	    }	      
-	} // if we found a class
-    }
-  return YES;
+  return result;
 }
 
 - (BOOL) isAction: (NSString *)name ofClass: (NSString *)className
