@@ -190,21 +190,6 @@
 	  [panel orderFront: self];
 	}
     }
-  else if ([name isEqual: NSWindowDidResignKeyNotification] == YES)
-    {
-      if (current == 1)
-	{
-	  /* FIXME - need to fix window focus handling for this to work */
-	  // [NSApp stopConnecting];
-	}
-    }
-  else if ([name isEqual: IBWillCloseDocumentNotification] == YES)
-    {
-      // FIXME
-      // show an empty selection of the document closes
-      // [self setEmptyInspector];
-      // [panel orderOut: self];
-    }
 }
 
 - (id) init
@@ -319,22 +304,11 @@
       selector: @selector(handleNotification:)
       name: IBWillEndTestingInterfaceNotification
       object: nil];
-  [nc addObserver: self
-      selector: @selector(handleNotification:)
-      name: NSWindowDidResignKeyNotification
-      object: panel];
   [nc addObserver: self
       selector: @selector(updateInspectorPopUp:)
       name: NSPopUpButtonWillPopUpNotification
       object: popup];
-  [nc addObserver: self
-      selector: @selector(handleNotification:)
-      name: NSPopUpButtonWillPopUpNotification
-      object: popup];
-  [nc addObserver: self
-      selector: @selector(handleNotification:)
-      name: IBWillCloseDocumentNotification
-      object: [(id<IB>)NSApp activeDocument]];
+
   [popup setTarget: self];
   [popup setAction: @selector(updateInspectorPopUp:)];
   return self;
@@ -699,6 +673,20 @@
     }
 }
 
+- (void) _selectAction: (NSString *)action
+{
+  /*
+   * Ensure that the actions are displayed in column one,
+   * and select the action for the current connection (if any).
+   */
+  [newBrowser reloadColumn: 1];
+  if (action != nil)
+    {
+      [newBrowser selectRow: [actions indexOfObject: action]
+		  inColumn: 1];
+    }
+}
+
 - (void) _internalCall: (NSBrowser*)sender
 {
   unsigned	numConnectors = [connectors count];
@@ -714,7 +702,6 @@
 	  if ([title isEqual: @"target"])
 	    {
 	      id	con = nil;
-	      NSString	*action;
 
 	      for (index = 0; index < numConnectors; index++)
 		{
@@ -732,7 +719,7 @@
 		    }
 		}
 
-	      if (con == nil)
+	      if (con == nil) // && [actions containsObject: [currentConnector label]] == NO) 
 		{
 		  RELEASE(actions);
 		  actions = RETAIN([[NSApp classManager]
@@ -747,21 +734,17 @@
 		    }
 		}
 
+	      // if we changed the current connector, update to the new one...
   	      if (currentConnector != con)
   		{
   		  ASSIGN(currentConnector, con);
   		}
+
 	      /*
 	       * Ensure that the actions are displayed in column one,
 	       * and select the action for the current connection (if any).
 	       */
-	      [newBrowser reloadColumn: 1];
-  	      action = [con label];
-  	      if (action != nil)
-  		{
-  		  [newBrowser selectRow: [actions indexOfObject: action]
-  			       inColumn: 1];
-  		}
+	      [self _selectAction: [con label]];
 	    }
 	  else
 	    {
@@ -989,8 +972,7 @@ selectCellWithString: (NSString*)title
 {
   // got the notification...  since we only subscribe to one, just do what
   // needs to be done.
-  [self setObject: object];
-  [self _internalCall: newBrowser]; // reload the connections browser..  
+  [self setObject: object]; // resets the browser...
 }
 
 - (id) init
@@ -998,7 +980,6 @@ selectCellWithString: (NSString*)title
   self = [super init];
   if (self != nil)
     {
-      NSNotificationCenter	*nc = [NSNotificationCenter defaultCenter];
       NSView		*contents;
       NSSplitView	*split;
       NSRect		rect;
@@ -1053,12 +1034,6 @@ selectCellWithString: (NSString*)title
       [revertButton setTarget: self];
       [revertButton setTitle: _(@"Revert")];
       [revertButton setEnabled: NO];
-
-      // catch notifications concerning connection deletions...
-      [nc addObserver: self 
-	  selector: @selector(handleNotification:)
-	  name: IBDidRemoveConnectorNotification
-	  object: nil];
     }
   return self;
 }
@@ -1074,17 +1049,20 @@ selectCellWithString: (NSString*)title
     }
   else if ([connectors containsObject: currentConnector] == YES)
     {
-      [[(id<IB>)NSApp activeDocument] removeConnector: currentConnector];
-      if ([currentConnector isKindOfClass: [NSNibOutletConnector class]])
+      id con = currentConnector;
+      RETAIN(con);
+      [[(id<IB>)NSApp activeDocument] removeConnector: con];
+      if ([con isKindOfClass: [NSNibOutletConnector class]])
 	{
-	  [currentConnector setDestination: nil];
+	  [con setDestination: nil];
 	}
-      if ([currentConnector isKindOfClass: [NSNibControlConnector class]])
+      if ([con isKindOfClass: [NSNibControlConnector class]])
 	{
-	  [currentConnector setDestination: nil];
-	  [currentConnector setLabel: nil];
+	  [con setDestination: nil];
+	  [con setLabel: nil];
 	}
-      [connectors removeObject: currentConnector];
+      [connectors removeObject: con];
+      RELEASE(con);
       [oldBrowser loadColumnZero];
     }
   else
@@ -1106,16 +1084,17 @@ selectCellWithString: (NSString*)title
 	      if ([con isKindOfClass: [NSNibControlConnector class]])
 		{
 		  [[(id<IB>)NSApp activeDocument] removeConnector: con];
-		  [con setDestination: nil];
-		  [con setLabel: nil];
 		  [connectors removeObjectIdenticalTo: con];
 		  break;
 		}
 	    }
+
+	  // select the new action from the list...
+	  [self _selectAction: [currentConnector label]];
 	}
       [connectors addObject: currentConnector];
       [[(id<IB>)NSApp activeDocument] addConnector: currentConnector];
-
+      
       /*
        * When we establish a connection, we want to highlight it in
        * the browser so the user can see it has been done.
@@ -1138,7 +1117,7 @@ selectCellWithString: (NSString*)title
       NSArray		*array;
 
       [super setObject: anObject];
-      DESTROY(currentConnector);
+      // DESTROY(currentConnector);
       RELEASE(connectors);
 
       /*
