@@ -62,6 +62,20 @@ static NSImage *horizontalImage;
       return @"GormViewEditor";
     }
 }
+
+- (NSImage*) imageForViewer
+{
+  static NSImage	*image = nil;
+
+  if (image == nil)
+    {
+      NSBundle	*bundle = [NSBundle mainBundle];
+      NSString *path = [bundle pathForImageResource: @"GormView"]; 
+      image = [[NSImage alloc] initWithContentsOfFile: path];
+    }
+
+  return image;
+}
 @end
 
 
@@ -875,7 +889,6 @@ static NSImage *horizontalImage;
 		    [NSArray arrayWithObject: editor]];
 	  }
 	}
-      // FIXME  we should maybe open ourself
     }
 
   return YES;
