@@ -30,18 +30,14 @@
 #include "GormViewWithSubviewsEditor.h"
 #include "GormPlacementInfo.h"
 #include "GormFunctions.h"
+#include "GormViewWindow.h"
 
 #include <math.h>
 #include <stdlib.h>
 
-
-
-
 @implementation GormPlacementInfo
 @end
 
-
-
 @implementation GormPlacementHint
 - (float) position { return _position; }
 - (float) start { return _start; }
@@ -140,7 +136,14 @@ static BOOL currently_displaying = NO;
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
   [NSException raise: NSInternalInconsistencyException
-	      format: @"Argh - encoding view editor"];
+	      format: @"Cannot encode a GormViewEditor."];
+}
+
+- (id) initWithCoder: (NSCoder*)aCoder
+{
+  [NSException raise: NSInternalInconsistencyException
+	      format: @"Cannot decode a GormViewEditor."];
+  return nil;
 }
 
 - (id<IBDocuments>) document
@@ -159,7 +162,22 @@ static BOOL currently_displaying = NO;
 {
   if (activated == NO)
     {
-      NSView *superview = [_editedObject superview];
+      NSView *superview;
+
+      // put the view into a "view window" if it's a standalone view...
+      if([_editedObject window] == nil)
+	{
+	  if(viewWindow == nil)
+	    {
+	      viewWindow = [[GormViewWindow alloc] initWithView: _editedObject];
+	    }
+	  else
+	    {
+	      [viewWindow setView: _editedObject];
+	    }
+	}
+
+      superview = [_editedObject superview];
 
       [self setFrame: [_editedObject frame]];
       [self setBounds: [self frame]];
@@ -168,8 +186,16 @@ static BOOL currently_displaying = NO;
 		 with: self];
       [self setAutoresizingMask: NSViewMaxXMargin | NSViewMinYMargin];
 
-      [self setAutoresizesSubviews: NO];
-
+      // we want autoresizing for standalone views...
+      if(viewWindow == nil)
+	{
+	  [self setAutoresizesSubviews: NO];
+	}
+      else
+	{
+	  [self setAutoresizesSubviews: YES];
+	}
+      
       [self addSubview: _editedObject];
       
       [_editedObject setPostsFrameChangedNotifications: YES];
@@ -187,13 +213,18 @@ static BOOL currently_displaying = NO;
 	object: self];
 
       parent = (GormViewWithSubviewsEditor *)[document parentEditorForEditor: self];
-
       if ([parent isKindOfClass: [GormViewEditor class]])
-	[parent setNeedsDisplay: YES];
+	{
+	  [parent setNeedsDisplay: YES];
+	}
       else
-	[self setNeedsDisplay: YES];
+	{
+	  [self setNeedsDisplay: YES];
+	}
+
       activated = YES;
-      return YES;
+
+      return activated;
     }
   
   return NO;
@@ -216,7 +247,12 @@ static BOOL currently_displaying = NO;
   if (closed == NO)
     {
       [self deactivate];
-      
+
+      if(viewWindow != nil)
+	{
+	  [viewWindow close];
+	}
+
       [document editor: self didCloseForObject: _editedObject];
       closed = YES;
     }
@@ -238,6 +274,13 @@ static BOOL currently_displaying = NO;
 
       [[NSNotificationCenter defaultCenter] removeObserver: self];
 
+      // make sure the view isn't in the window after deactivation.
+      if(viewWindow != nil)
+	{
+	  [_editedObject removeFromSuperview]; // WithoutNeedingDisplay];
+	  [viewWindow orderOut: self];
+	}
+
       activated = NO;
     }
 }
@@ -1236,7 +1279,12 @@ static BOOL currently_displaying = NO;
     {
       if (parent)
 	{
-	  [parent mouseDown: theEvent];
+	  // TODO: We should find a better test than this, but it will do
+	  // for now...
+	  if([parent isKindOfClass: [GormGenericEditor class]] == NO)
+	    {
+	      [parent mouseDown: theEvent];
+	    }
 	}
       else
 	return [self noResponderFor: @selector(mouseDown:)];
@@ -1316,7 +1364,19 @@ static BOOL currently_displaying = NO;
 
 - (void) resetObject: (id)anObject
 {
-  NSLog(@"resetObject should not be called on GormViewEditor !");
+  NS_DURING
+    {
+      // display the view, if it's standalone.
+      if(viewWindow != nil)
+	{
+	  [viewWindow orderFront: self];
+	}
+    }
+  NS_HANDLER
+    {
+      NSLog(@"Exception while trying to display standalone view: %@",[localException reason]);
+    }
+  NS_ENDHANDLER
 }
 
 - (void) orderFront
