@@ -156,6 +156,11 @@
   return NO;
 }
 
+- (void) rightMouseDown: (NSEvent*)theEvent
+{
+  // Do nothing.  We want to ignore when the right mouse button is pressed.
+}
+
 - (void) mouseDown: (NSEvent*)theEvent
 {
   NSPoint	loc = [theEvent locationInWindow];
@@ -513,6 +518,7 @@
       [self close];
     }
 
+  RELEASE(edited);
   RELEASE(selection);
   RELEASE(subeditor);
   [super dealloc];
@@ -678,7 +684,6 @@ void _attachAll(NSMenu *menu, id document)
     {
       document = aDocument;
       ASSIGN(edited, anObject);
-      RETAIN(edited);
       selection = [NSMutableArray new];
       rep = [edited menuRepresentation];
       
@@ -848,6 +853,12 @@ void _attachAll(NSMenu *menu, id document)
 	      NSDebugLog(@"owned by popup");
 	      [item setOnStateImage: nil];
 	      [item setMixedStateImage: nil];
+
+	      // if the item has a submenu, reject the drag.
+	      if([item hasSubmenu])
+		{
+		  return NO;
+		}
 	    }
 	  else
 	    NSDebugLog(@"not owned by popup");
@@ -1076,8 +1087,6 @@ static BOOL done_editing;
   didDrawBackground = [editField drawsBackground];
   [editField setDrawsBackground: YES];
 
-//    [editField display];
-
   [nc addObserver: self
          selector: @selector(handleNotification:)
              name: NSControlTextDidEndEditingNotification
