@@ -144,7 +144,8 @@
   id typeButton;
   id keyEquiv;
 }
-
+- (void) setButtonType: (NSButtonType)type forObject: (id)button;
+- (void) _setValuesFromControl: (id)control;
 - (void) _getValuesFromObject: (id)anObject;
 @end
 
@@ -225,8 +226,7 @@
       BOOL flag;
 
       flag = ([[control cellAtRow: 0 column: 0] state] == NSOnState) ? YES : NO;
-      [object setBordered: flag];
-      flag = ([[control cellAtRow: 1 column: 0] state] == NSOnState) ? YES : NO;
+      [object setBordered: flag];      flag = ([[control cellAtRow: 1 column: 0] state] == NSOnState) ? YES : NO;
       [object setContinuous: flag];
       flag = ([[control cellAtRow: 2 column: 0] state] == NSOnState) ? YES : NO;
       [object setEnabled: flag];
@@ -243,13 +243,14 @@
     {
       NSString *string;
       NSImage *image;
-
+      
       [object setTitle: [[control cellAtIndex: 0] stringValue]];
       [object setAlternateTitle: [[control cellAtIndex: 1] stringValue]];
+
       string = [[control cellAtIndex: 2] stringValue];
       if ([string length] > 0)
 	{   
-          image = [NSImage imageNamed: string];
+	  image = [NSImage imageNamed: string];
 	  [object setImage: image];
 	}
       string = [[control cellAtIndex: 3] stringValue];
@@ -379,25 +380,17 @@
   /* Doesn't work yet? */
   //  [typeButton setAction: @selector(setButtonTypeFrom:)];
   //  [typeButton setTarget: self];
-
+ 
   return self;
 }
 
 - (void) ok: (id)sender
 {
-  NSDebugLog(@"ok: sender = %@",sender);
   [self _setValuesFromControl: sender];
 }
 
 - (void) setObject: (id)anObject
 {
-  if ([self object] != anObject)
-    {
-      /*
-       * Ensure textfields in title form are written to old object.
-       */
-      [self _setValuesFromControl: titleForm];
-    }
   [super setObject: anObject];
   [self _getValuesFromObject: anObject];
 }
@@ -860,7 +853,8 @@
 @interface GormPopUpButtonAttributesInspector : IBInspector
 {
   id typeMatrix;
-  id disabledSwitch;
+  id autoenableSwitch;
+  id enableSwitch;
   id tagForm;
   id defaultItemForm;
 }
@@ -878,9 +872,13 @@
       [object selectItem: nil];
       [object selectItem: selectedItem];
     }
-  else if (control == disabledSwitch)
+  else if (control == autoenableSwitch)
     {
-      [object setAutoenablesItems: ([control state] == NSOffState)];
+      [object setAutoenablesItems: ([control state] == NSOnState)];
+    }
+  else if (control == enableSwitch)
+    {
+      [object setEnabled: ([control state] == NSOnState)];
     }
   else if (control == tagForm)
     {
@@ -904,7 +902,8 @@
       return;
     }
   [typeMatrix selectCellWithTag: [anObject pullsDown]];
-  [disabledSwitch setState: ![anObject autoenablesItems]];
+  [autoenableSwitch setState: [anObject autoenablesItems]];
+  [enableSwitch setState: [anObject isEnabled]];
   [[tagForm cellAtRow: 0 column: 0] setIntValue: [anObject tag]];
   [[defaultItemForm cellAtRow: 0 column: 0] setIntValue: [anObject indexOfSelectedItem]];
 }
