@@ -3,7 +3,8 @@
    Copyright (C) 2001 Free Software Foundation, Inc.
 
    Author:  Laurent Julliard <laurent@julliard-online.org>
-   Date: Aug 2001
+   Author:  Gregory John Casamento <greg_casamento@yahoo.com>
+   Date: Aug 2001. 2003, 2004
    
    This file is part of GNUstep.
    
@@ -24,9 +25,9 @@
 
 #include <Foundation/Foundation.h>
 #include <AppKit/AppKit.h>
+#include <InterfaceBuilder/InterfaceBuilder.h>
 #include "GormNSTableView.h"
-#include <InterfaceBuilder/IBInspector.h>
-#include <InterfaceBuilder/IBObjectAdditions.h>
+#include "GormPrivate.h"
 
 /* This macro makes sure that the string contains a value, even if @"" */
 #define VSTR(str) ({id _str = str; (_str) ? _str : @"";})
@@ -184,6 +185,9 @@
   id identifierTextField;
   id resizableSwitch;
   id editableSwitch;
+  id setButton;
+  id defaultButton;
+  id cellTable;
 }
 - (void) _getValuesFromObject: (id)anObject;
 - (void) _setValuesFromControl: (id)anObject;
@@ -206,6 +210,11 @@
   return self;
 }
 
+- (void) awakeFromNib
+{
+  [cellTable setDoubleAction: @selector(ok:)];
+}
+
 - (void) ok: (id)sender
 {
   [self _setValuesFromControl: sender];
@@ -216,8 +225,37 @@
   [super setObject: anObject];
   [self _getValuesFromObject: anObject];
 }
+
+- (NSString *)_getCellClassName
+{
+  id cell = [[self object] dataCell];
+  NSString *customClassName = [[(Gorm *)NSApp classManager] customClassForObject: cell];
+  NSString *result = nil;
+
+  if(customClassName == nil)
+    {
+      result = NSStringFromClass([cell class]);
+    }
+  else
+    {
+      result = customClassName;
+    }
+  
+  return result;
+}
+
 - (void) _getValuesFromObject: anObject
 {
+  NSArray *list = [[(Gorm *)NSApp classManager] allSubclassesOf: @"NSCell"];
+  NSString *cellClassName = [self _getCellClassName];
+  int index = [list indexOfObject: cellClassName];
+
+  if(index != NSNotFound && index != -1)
+    {
+      [cellTable selectRow: index byExtendingSelection: NO];
+      [cellTable scrollRowToVisible: index];
+    }
+  
   switch ([[anObject headerCell] alignment])
     {
     case NSLeftTextAlignment:
@@ -261,6 +299,7 @@
     [editableSwitch setState: NSOnState];
   else
     [editableSwitch setState: NSOffState];
+
 }
 
 - (void) _setValuesFromControl: (id) control
@@ -316,6 +355,89 @@
       [object setResizable:
 		([resizableSwitch state] == NSOnState)];
     }
+  else if (control == setButton || control == cellTable)
+    {
+      id classManager = [(Gorm *)NSApp classManager];
+      id<IBDocuments> doc = [(id<IB>)NSApp activeDocument];
+      id cell = nil;
+      int i = [cellTable selectedRow];
+      NSArray *list = [classManager allSubclassesOf: @"NSCell"];
+      NSString *className = [list objectAtIndex: i];
+      BOOL isCustom = [classManager isCustomClass: className];
+      Class cls = nil;
+
+      if(isCustom)
+	{
+	  NSString *superClass = [classManager nonCustomSuperClassOf: className];
+	  cls = NSClassFromString(superClass);
+	  NSLog(@"Setting custom cell..");
+	}
+      else
+	{
+	  cls = NSClassFromString(className);
+	}
+
+      // initialize
+      cell = [cls new];
+      [object setDataCell: cell];
+      [[object tableView] setNeedsDisplay: YES];
+
+      // add it to the document, since it needs a custom class...
+      if(isCustom)
+	{
+	  NSString *name = nil;
+
+	  // An object needs to be a "named object" to have a custom class
+	  // assigned to it.   Add it to the document and get the name.
+	  [doc attachObject: cell toParent: object];
+	  if((name = [doc nameForObject: cell]) != nil)
+	    {
+	      [classManager setCustomClass: className forObject: name];
+	    } 
+	}
+
+      RELEASE(cell);
+    }
+  else if (control == defaultButton)
+    {
+      [object setDataCell: [[NSTextFieldCell alloc] init]];
+      [[object tableView] setNeedsDisplay: YES];
+      [self setObject: [self object]]; // reset...
+    }
+}
+
+// data source
+- (int) numberOfRowsInTableView: (NSTableView *)tv
+{
+  NSArray *list = [[(Gorm *)NSApp classManager] allSubclassesOf: @"NSCell"];
+  return [list count];
+}
+
+- (id)          tableView: (NSTableView *)tv
+objectValueForTableColumn: (NSTableColumn *)tc
+	              row: (int)rowIndex
+{
+  NSArray *list = [[(Gorm *)NSApp classManager] allSubclassesOf: @"NSCell"];
+  id value = nil;
+  if([list count] > 0)
+    {
+      value = [list objectAtIndex: rowIndex];
+    }
+  return value;
+}
+
+// delegate
+- (BOOL)    tableView: (NSTableView *)tableView
+shouldEditTableColumn: (NSTableColumn *)aTableColumn
+		  row: (int)rowIndex
+{
+  return NO;
+}
+
+- (BOOL) tableView: (NSTableView *)tv
+   shouldSelectRow: (int)rowIndex
+{
+  return YES;
 }
 @end
 
@@ -483,8 +605,8 @@
 	      // remove columns...
 	      for(i = 0; i < colsToDelete; i++)
 		{
-	      NSTableColumn *tc = [columns objectAtIndex: (i + newNumCols)];
-	      [object removeTableColumn: tc];
+		  NSTableColumn *tc = [columns objectAtIndex: (i + newNumCols)];
+		  [object removeTableColumn: tc];
 		}
 	    }
 	}
@@ -512,7 +634,6 @@
     }
 
   [scrollView setNeedsDisplay: YES];
-
 }
 
 - (void) _getValuesFromObject: anObject
@@ -520,8 +641,7 @@
   BOOL isScrollView;
   id scrollView;
 
-  scrollView = //[[object superview] superview];
-    [object enclosingScrollView];
+  scrollView = [object enclosingScrollView];
 
   isScrollView = [ scrollView isKindOfClass: [NSScrollView class]];
 
@@ -603,37 +723,6 @@
 
 @end
 
-
-/*----------------------------------------------------------------------------
- * NSScrollView
- */
-
-/*
- * For now NSScrollView has no inspector in itself. It is only used as a 
- * convenience in the NSTableView and NSTextView controls and there
- * are minimal NSScrollView settings in the inspector of these 2 controls
- * (like horizontal and vertical scrollbar)
-*/
-
-/*
-@implementation	NSScrollView (IBObjectAdditions)
-
-- (NSString*) inspectorClassName
-{
-  return @"GormEmptyInspector";
-}
-
-@end
-
-@interface GormScrollViewAttributesInspector : IBInspector
-{
-}
-@end
-
-@implementation GormScrollViewAttributesInspector
-@end
-*/
-
 /*----------------------------------------------------------------------------
  * NSTabView (possibly embedded in a Scroll view)
  */
